@@ -122,7 +122,7 @@ int	logging;
 int	guest;
 #ifdef STATS
 int	stats;		  /* gather stats about which files goes to whom  */
-int	statfd = -1;
+FILE	*statfp = 0;
 #endif
 int	type;
 int	form;
@@ -283,8 +283,10 @@ nextopt:
 		argc--, argv++;
 	}
 	(void) freopen("/dev/null", "w", stderr);
+	(void) signal(SIGPIPE, lostconn);
 	(void) signal(SIGCHLD, SIG_IGN);
-	enable_signalling();
+	if ((int)signal(SIGURG, myoob) < 0)
+		syslog(LOG_ERR, "signal: %m");
 
 	/* avoid core dumps with cleartext or (shadow) passwords. */
 #ifdef RLIMIT_CORE
@@ -321,20 +323,6 @@ nextopt:
 	/* NOTREACHED */
 }
 
-enable_signalling()
-{
-	(void) signal(SIGPIPE, lostconn);
-	if ((int)signal(SIGURG, myoob) < 0)
-		syslog(LOG_ERR, "signal: %m");
-}
-
-disable_signalling()
-{
-	(void) signal(SIGPIPE, SIG_IGN);
-	if ((int)signal(SIGURG, SIG_IGN) < 0)
-		syslog(LOG_ERR, "signal: %m");
-}
-
 void
 lostconn()
 {
@@ -460,7 +448,7 @@ user(name)
 		if (cp == NULL) {
 			reply(530, "User %s access denied.", name);
 				syslog(LOG_NOTICE,
-				    "FTP LOGIN REFUSED FROM %s, %s",
+				    "FTP LOGIN REFUSED FROM %.100s, %s",
 				    remotehost, name);
 			pw = (struct passwd *) NULL;
 			return;
@@ -472,7 +460,7 @@ user(name)
 			if (strcmp(line, name) == 0) {
 				reply(530, "User %s access denied.", name);
 					syslog(LOG_NOTICE,
-					    "FTP LOGIN REFUSED FROM %s, %s",
+					    "FTP LOGIN REFUSED FROM %.100s, %s",
 					    remotehost, name);
 				pw = (struct passwd *) NULL;
 				return;
@@ -504,20 +492,21 @@ user(name)
 end_login()
 {
 
-	disable_signalling();
+	delay_signaling();
 	(void) seteuid((uid_t)0);
 	if (logged_in)
 		logwtmp(ttyline, "", "");
 	pw = NULL;
 	logged_in = 0;
 	guest = 0;
-	enable_signalling();
+	enable_signaling();
 }
 
 pass(passwd)
 	char *passwd;
 {
 	char *xpasswd, *salt;
+	int fd;
 
 	if (logged_in || askpasswd == 0) {
 		reply(503, "Login with USER first.");
@@ -542,7 +531,7 @@ pass(passwd)
 			last.failures++;
 			if (login_attempts++ >= 5) {
 				syslog(LOG_ERR,
-				    "REPEATED LOGIN FAILURES FROM %s, %s",
+				    "REPEATED LOGIN FAILURES FROM %.100s, %s",
 				    remotehost, last.name);
 				exit(0);
 			}
@@ -551,7 +540,7 @@ pass(passwd)
 	}
 	login_attempts = 0;		/* this time successful */
 	last.failures = 0;
-	syslog(LOG_INFO, "login from %s as %s", remotehost, pw->pw_name);
+	syslog(LOG_INFO, "login from %.100s as %s", remotehost, pw->pw_name);
 	(void) setegid((gid_t)pw->pw_gid);
 	(void) initgroups(pw->pw_name, pw->pw_gid);
 
@@ -561,8 +550,9 @@ pass(passwd)
 	logged_in = 1;
 #ifdef STATS
 	/* and stat file */
-	if (guest && stats && statfd < 0)
-		if ((statfd = open(STATFILE, O_WRONLY|O_APPEND)) < 0)
+	if (guest && stats && statfp == 0)
+		if ((fd = open(STATFILE, O_WRONLY|O_APPEND)) < 0
+		|| (statfp = fdopen(fd, "a")) == 0)
 			stats = 0;
 #endif
 	if (guest) {
@@ -575,17 +565,23 @@ pass(passwd)
 			reply(550, "Can't set guest privileges.");
 			goto bad;
 		}
-	} else if (chdir(pw->pw_dir) < 0) {
+		if (seteuid((uid_t)pw->pw_uid) < 0) {
+			reply(550, "Can't set uid.");
+			goto bad;
+		}
+	} else {
+	    if (seteuid((uid_t)pw->pw_uid) < 0) {
+		    reply(550, "Can't set uid.");
+		    goto bad;
+	    }
+	    if (chdir(pw->pw_dir) < 0) {
 		if (chdir("/") < 0) {
 			reply(530, "User %s: can't change directory to %s.",
 			    pw->pw_name, pw->pw_dir);
 			goto bad;
 		} else
 			lreply(230, "No directory! Logging in with home=/");
-	}
-	if (seteuid((uid_t)pw->pw_uid) < 0) {
-		reply(550, "Can't set uid.");
-		goto bad;
+	    }
 	}
 	if (guest) {
 #ifndef MOTD
@@ -622,7 +618,7 @@ pass(passwd)
 		ident = copy(passwd);
 #endif
 #ifdef SETPROCTITLE
-		sprintf(proctitle, "%s: anonymous/%.*s", remotehost,
+		sprintf(proctitle, "%.100s: anonymous/%.*s", remotehost,
 		    sizeof(proctitle) - sizeof(remotehost) -
 		    sizeof(": anonymous/"), passwd);
 		setproctitle(proctitle);
@@ -630,7 +626,7 @@ pass(passwd)
 	} else {
 		reply(230, "User %s logged in.", pw->pw_name);
 #ifdef SETPROCTITLE
-		sprintf(proctitle, "%s: %s", remotehost, pw->pw_name);
+		sprintf(proctitle, "%.100s: %s", remotehost, pw->pw_name);
 		setproctitle(proctitle);
 #endif /* SETPROCTITLE */
 	}
@@ -684,7 +680,7 @@ retrieve(cmd, name)
 	send_data(fin, dout, st.st_blksize);
 #ifdef STATS
 	if (cmd == 0 && guest && stats)
-		logxfer(name, st.st_size, start);
+		logxfer(name, (long) st.st_size, start);
 #endif
 	(void) fclose(dout);
 	data = -1;
@@ -737,7 +733,7 @@ getdatasock(mode)
 
 	if (data >= 0)
 		return (fdopen(data, mode));
-	disable_signalling();
+	delay_signaling();
 	(void) seteuid((uid_t)0);
 	s = socket(AF_INET, SOCK_STREAM, 0);
 	if (s < 0)
@@ -755,13 +751,13 @@ getdatasock(mode)
 		goto bad;
 	}
 	(void) seteuid((uid_t)pw->pw_uid);
-	enable_signalling();
+	enable_signaling();
 	return (fdopen(s, mode));
 bad:
 	(void) close(s);
 bad2:
 	(void) seteuid((uid_t)pw->pw_uid);
-	enable_signalling();
+	enable_signaling();
 	return (NULL);
 }
 
@@ -1261,8 +1257,9 @@ dolog(sin)
 	else
 		(void) strncpy(remotehost, inet_ntoa(sin->sin_addr),
 		    sizeof (remotehost));
+	remotehost[sizeof(remotehost) - 1] = 0;
 #ifdef SETPROCTITLE
-	sprintf(proctitle, "%s: connected", remotehost);
+	sprintf(proctitle, "%.100s: connected", remotehost);
 	setproctitle(proctitle);
 #endif /* SETPROCTITLE */
 }
@@ -1274,7 +1271,7 @@ dolog(sin)
 dologout(status)
 	int status;
 {
-	disable_signalling();
+	delay_signaling();
 	if (logged_in) {
 		(void) seteuid((uid_t)0);
 		logwtmp(ttyline, "", "");
@@ -1576,25 +1573,24 @@ char *fmt;
 logxfer(name, size, start)
 	char *name;
 	long size;
-	long start;
+	time_t start;
 {
-	char buf[1024];
 	char path[MAXPATHLEN + 1];
-	long now;
+	time_t now;
 
-	if (statfd >= 0) {
+	if (statfp != 0) {
 		if (name[0] == '/') {
 			path[0] = 0;
 		} else if (getwd(path) != 0) {
 			strcat(path, "/");
 		} else {
-			return;
+			strcpy(path, "?/");
 		}
 		time(&now);
-		sprintf(buf, "%.20s!%s!%s!%s%s!%ld!%ld\n",
+		fprintf(statfp, "%.20s!%s!%s!%s%s!%ld!%ld\n",
 			ctime(&now)+4, ident, remotehost,
-			path, name, size, now - start + (now == start));
-		write(statfd, buf, strlen(buf));
+			path, name, size, (long)(now - start + (now == start)));
+		fflush(statfp);
 	}
 }
 #endif
@@ -1604,7 +1600,7 @@ logxfer(name, size, start)
 badlogin()
 {
 	if (last.failures > 0 && last.failures > last.minfail)
-		syslog(LOG_ERR, "%d FTP LOGIN FAILURE%s from %s, %s",
+		syslog(LOG_ERR, "%d FTP LOGIN FAILURE%s from %.100s, %s",
 		       last.failures, last.failures > 1 ? "S" : "",
 		       remotehost, last.name);
 }
