@@ -12,7 +12,7 @@ char   *name;
 struct passwd *pwd;
 int     pwok;
 {
-    static char buf[128];
+    static char buf[BUFSIZ];
     struct skey skey;
 
     /* Display s/key challenge where appropriate. */
