@@ -119,6 +119,7 @@
 #define LOGIN_OPT_R			/* login -l option already taken */
 #define IRIX_LASTLOGIN			/* no holes in their file system? */
 #define HAS_PATHS_H			/* /usr/include/paths.h */
+#define ENV_LOGNAME			/* login sets LOGNAME */
 #endif
 
 #ifdef IRIX4
@@ -133,6 +134,7 @@
 #define _PATH_LOGIN "/bin/login"	/* not /usr/bin/login */
 #define NO_SETENV
 #define NO_ULIMIT_H			/* no <ulimit.h> file */
+#define ENV_LOGNAME			/* login sets LOGNAME */
 #endif
 
 #ifdef LINUX
@@ -142,6 +144,7 @@
 #define HAS_PATHS_H			/* /usr/include/paths.h */
 #define CONSOLE		"tty1"		/* no /dev/console */
 #define NBBY		8		/* not in <sys/mumble.h> */
+#define STDIO_DECLARES_SYS_ERRLIST	/* stdio.h declares sys_errlist */
 #endif
 
 #ifdef DECOSF1
@@ -170,13 +173,11 @@
 #define UTMP_INIT	utmpx_init
 #define UTMP_LOGIN	utmpx_login
 #define UTMP_LOGOUT	utmpx_logout
-#define MAKE_UTMP_ID	utmpx_ptsid
 #else
 #define UTMP_STRUCT	utmp
 #define UTMP_INIT	utmp_init
 #define UTMP_LOGIN	utmp_login
 #define UTMP_LOGOUT	utmp_logout
-#define MAKE_UTMP_ID	utmp_ptsid
 #endif
 
 #ifdef DES_RPC
