@@ -49,6 +49,8 @@ static char sccsid[] = "@(#)rlogind.c	5.22.1.6 (Berkeley) 2/7/89";
 #include <sys/file.h>
 
 #include <netinet/in.h>
+#include <netinet/in_systm.h>
+#include <netinet/ip.h>
 
 #include <errno.h>
 #include <pwd.h>
@@ -262,15 +264,44 @@ doit(f, fromp)
 		ipproto = IPPROTO_IP;
 	if (getsockopt(0, ipproto, IP_OPTIONS, (char *)optbuf, &optsize) == 0 &&
 	    optsize != 0) {
-		lp = lbuf;
-		for (cp = optbuf; optsize > 0; cp++, optsize--, lp += 3)
-			sprintf(lp, " %2.2x", *cp);
-		syslog(LOG_NOTICE,
-		    "Connection received using IP options (ignored):%s", lbuf);
-		if (setsockopt(0, ipproto, IP_OPTIONS,
-		    (char *)NULL, optsize) != 0) {
-			syslog(LOG_ERR, "setsockopt IP_OPTIONS NULL: %m");
-			exit(1);
+		int	source_routed = 0;
+		int	optlen;
+		int	opt;
+		struct in_addr dummy;
+
+#define ADDR_LEN sizeof(dummy.s_addr)
+
+		/*
+		 * Spot any nasty IP source-routing options.
+		 */
+		for (cp = optbuf + ADDR_LEN; cp < optbuf + optsize; cp += optlen) {
+		    opt = cp[IPOPT_OPTVAL];
+		    if (opt == IPOPT_LSRR || opt == IPOPT_SSRR)
+			source_routed = 1;
+		    if (opt == IPOPT_EOL)
+			break;
+		    if (opt == IPOPT_NOP) {
+			optlen = 1;
+		    } else {
+			optlen = cp[IPOPT_OLEN];
+			if (optlen <= 0)
+			    break;
+		    }
+		}
+		/*
+		 * Stomp on the options, but only if there were known nasties.
+		 */
+		if (source_routed) {
+			lp = lbuf;
+			for (cp = optbuf; optsize > 0; cp++, optsize--, lp += 3)
+				sprintf(lp, " %2.2x", *cp);
+			syslog(LOG_NOTICE,
+			    "Connection received using IP options (ignored):%s", lbuf);
+			if (setsockopt(0, ipproto, IP_OPTIONS,
+			    (char *)NULL, optsize) != 0) {
+				syslog(LOG_ERR, "setsockopt IP_OPTIONS NULL: %m");
+				exit(1);
+			}
 		}
 	}
       }
@@ -377,10 +408,8 @@ gotpty:
 		dup2(t, 0), dup2(t, 1), dup2(t, 2);
 		close(t);
 #ifdef SYSV_UTMP
-		{
-		char *MAKE_UTMP_ID(), *utmp_id = MAKE_UTMP_ID(line, "rl");
-		UTMP_INIT(line + sizeof("/dev/") - 1, ".rlogin", utmp_id);
-		}
+		if (UTMP_INIT(line + sizeof("/dev/") - 1, ".rlogin", "rl") < 0)
+			fatalperror(2, "makeutx failed");
 #endif /* SYSV_UTMP */
 #ifdef OLD_LOGIN
 		execl(_PATH_LOGIN, "login", _check_rhosts_file ? "-r" : "-lr",
@@ -772,6 +801,7 @@ int     deny_severity = LOG_WARNING;
 
 do_access(request, host, ruser, luser)
 struct request_info *request;
+char   *host;
 char   *ruser;
 char   *luser;
 {
