@@ -64,7 +64,7 @@ extern	char	*ptsname();
 #include <sys/fcntl.h>
 #endif
 
-#ifndef FIONBIO
+#if !defined(FIONBIO) && !defined(DECOSF1)
 #include <sys/filio.h>
 #endif
 
@@ -431,10 +431,8 @@ gotpty:
 	 */
 #ifdef SYSV_UTMP
 	/* SYSV login insists on an utmp(x) entry */
-	{
-		char *MAKE_UTMP_ID(), *utmp_id = MAKE_UTMP_ID(line, "tn");
-		UTMP_INIT(line + sizeof("/dev/") - 1, ".telnet", utmp_id);
-	}
+	if (UTMP_INIT(line + sizeof("/dev/") - 1, ".telnet", "tn") < 0)
+		fatalperror(2, "makeutx failed");
 #endif
 #ifdef SYSV_ENV
 	execl(_PATH_LOGIN, "login", "-h", host,
